@@ -42,127 +42,43 @@
 #ifndef _basetype_h
 #define _basetype_h 1
 
-
 #include <vector>
 #include <stack>
 #include <iostream>
 #include <string>
 
-// These are instantiated only for DAP4 variables
-#if DAP4
-#include "D4Dimensions.h"
-#include "D4Maps.h"
-#endif
-
 #include "AttrTable.h"
 
 #include "InternalErr.h"
 
 #include "dods-datatypes.h"
+#include "Type.h"
 
 #include "DapObj.h"
 
-#include "XMLWriter.h"
-
 using namespace std;
 
+class Crc32;
+
 namespace libdap
 {
 
-class DDS;
 class ConstraintEvaluator;
+
+class DDS;
 class Marshaller;
 class UnMarshaller;
 
-/** <b>Part</b> names the parts of multi-section constructor types.
-    For example, the <b>Grid</b> class has an <i>array</i> and
-    the array <i>maps</i>. Use the <tt>nil</tt> value for data types that
-    don't have separate parts.
+class Constructor;
+class XMLWrter;
 
-    \code
-    enum Part {
-    nil,
-    array,
-    maps
-    };
-    \endcode
+class DMR;
+class D4Group;
+class XMLWriter;
+class D4StreamMarshaller;
+class D4StreamUnMarshaller;
 
-    @brief Names the parts of multi-section constructor data types.
-    @see Grid
-    @see BaseType
-*/
-
-enum Part {
-    nil,   // nil is for types that don't have parts...
-    array,
-    maps
-};
-
-/** <b>Type</b> identifies the data type stored in a particular type
-    class. All the DODS Data Access Protocol (DAP) types inherit from
-    the BaseType class.
-
-    \code
-    enum Type {
-    dods_null_c,
-    dods_byte_c,
-    dods_int16_c,
-    dods_uint16_c,
-    dods_int32_c,
-    dods_uint32_c,
-    dods_float32_c,
-    dods_float64_c,
-    dods_str_c,
-    dods_url_c,
-    dods_array_c,
-    dods_structure_c,
-    dods_sequence_c,
-    dods_grid_c,
-
-    dods_int8_c,
-    dods_uint8_c,
-    dods_int64_c,
-    dods_uint64_c,
-    dods_url4_c
-    dods_enum_c,
-    dods_group_c
-
-    };
-    \endcode
-
-    @brief Identifies the data type.
-    @see BaseType
-*/
-
-enum Type {
-    dods_null_c,
-    dods_byte_c,
-    dods_int16_c,
-    dods_uint16_c,
-    dods_int32_c,  // Added `dods_' to fix clash with IRIX 5.3.
-    dods_uint32_c,
-    dods_float32_c,
-    dods_float64_c,
-    dods_str_c,
-    dods_url_c,
-    dods_array_c,
-    dods_structure_c,
-    dods_sequence_c,
-    dods_grid_c,
-
-    // Added for DAP4
-    dods_int8_c,
-    dods_uint8_c,
-
-    dods_int64_c,
-    dods_uint64_c,
-
-    dods_url4_c,
-
-    dods_enum_c,
-    dods_group_c
-
-};
+class D4Attributes;
 
 /** This defines the basic data type features for the DODS data access
     protocol (DAP) data types. All the DAP type classes (Float64, Array,
@@ -174,11 +90,11 @@ enum Type {
     access and use this data. On the server side, in many cases, the class
     instances will have no data in them at all until the
     <tt>serialize</tt> function
-    is called to send data to the client. On the client side, most DODS
+    is called to send data to the client. On the client side, most DAP
     application programs will unpack the data promptly into whatever local
     data structure the programmer deems the most useful.
 
-    In order to use these classes on the server side of a DODS
+    In order to use these classes on the server side of a DAP
     client/server connection, you must write a <tt>read</tt> method
     for each of the data types you expect to encounter in the
     application. This function, whose purpose is to read data from a
@@ -194,6 +110,8 @@ enum Type {
     the constraints
     to be returned. These cautions are outlined where they occur.
 
+	@note This class is ued by both DAP2 and DAP4.
+
     @brief The basic data type for the DODS DAP types.  */
 
 class BaseType : public DapObj
@@ -205,8 +123,6 @@ private:
 
     bool d_is_read;  // true if the value has been read
     bool d_is_send;  // Is the variable in the projection?
-    bool d_in_selection; // Is the variable in the selection?
-    bool d_is_synthesized; // true if the variable is synthesized
 
     // d_parent points to the Constructor or Vector which holds a particular
     // variable. It is null for simple variables. The Vector and Constructor
@@ -216,17 +132,17 @@ private:
     // Attributes for this variable. Added 05/20/03 jhrg
     AttrTable d_attr;
 
+    D4Attributes *d_attributes;
+
     bool d_is_dap4;         // True if this is a DAP4 variable, false ... DAP2
 
     // These are non-empty only for DAP4 variables. Added 9/27/12 jhrg
 
-    // FIXME Remove this. This header cannot have compile-time variation
-#if DAP4
-    D4Dimensions d_dims;   // If non-empty, this BaseType is an DAP4 Array
-    D4Maps d_maps;         // if non-empty, this BaseType is a DAP4 'Grid'
-#endif
-
 protected:
+    // These were/are used for DAP2 CEs, but not for DAP4 ones
+    bool d_in_selection; // Is the variable in the selection?
+    bool d_is_synthesized; // true if the variable is synthesized
+
     void m_duplicate(const BaseType &bt);
 
 public:
@@ -235,22 +151,35 @@ public:
     // These ctors assume is_dap4 is false
     BaseType(const string &n, const Type &t, bool is_dap4 = false);
     BaseType(const string &n, const string &d, const Type &t, bool is_dap4 = false);
-#if 0
-    // These provide a way to set is_dap4
-    BaseType(const string &n, const Type &t, bool is_dap4);
-    BaseType(const string &n, const string &d, const Type &t, bool is_dap4);
-#endif
+
     BaseType(const BaseType &copy_from);
     virtual ~BaseType();
 
     virtual string toString();
 
+    virtual void transform_to_dap4(D4Group *root, Constructor *container);
+    virtual std::vector<BaseType *> *transform_to_dap2(AttrTable *parent_attr_table);
+
     virtual void dump(ostream &strm) const ;
 
     BaseType &operator=(const BaseType &rhs);
 
-    bool is_dap4() { return d_is_dap4; }
-    void set_is_dap4(const bool v) { d_is_dap4 = v;}
+    /**
+     * Remove any read or set data in the private data of the variable,
+     * setting read_p() to false. Used to clear any dynamically allocated
+     * storage that holds (potentially large) data. For the simple types,
+     * this no-op version is all that's needed. Vector and some other classes
+     * define a special version and have serialize() implementations that
+     * call it to free data as soon as possible after sending it.
+     *
+     * @note Added 7/5/15 jhrg
+     * @note Any specialization of this should make sure to reset the read_p
+     * property.
+     */
+    virtual void clear_local_data() { set_read_p(false); }
+
+    virtual bool is_dap4() const { return d_is_dap4; }
+    virtual void set_is_dap4(const bool v) { d_is_dap4 = v;}
 
     /** Clone this instance. Allocate a new instance and copy \c *this into
 	it. This method must perform a deep copy.
@@ -260,24 +189,34 @@ public:
 	@return A newly allocated copy of \c this. */
     virtual BaseType *ptr_duplicate() = 0;
 
-    string name() const;
+    virtual string name() const;
     virtual void set_name(const string &n);
+    virtual std::string FQN() const;
 
-    Type type() const;
-    void set_type(const Type &t);
-    string type_name() const;
+    virtual Type type() const;
+    virtual void set_type(const Type &t);
+    virtual string type_name() const;
 
-    string dataset() const ;
+    virtual string dataset() const ;
 
-    virtual bool is_simple_type();
-    virtual bool is_vector_type();
-    virtual bool is_constructor_type();
+    /**
+     * @brief How many elements are in this variable.
+     * @todo change the return type to int64_t
+     * @return The number of elements; 1 for scalars
+     */
+    virtual int length() const { return 1; }
+
+    /**
+     * @brief Set the number of elements for this variable
+     * @todo change param type to int64_t
+     * @param l The number of elements
+     */
+    virtual void set_length(int) { }
+
+    virtual bool is_simple_type() const;
+    virtual bool is_vector_type() const;
+    virtual bool is_constructor_type() const;
 
-#if 0
-    // Not yet, if ever. Allow 'sloppy' changeover in the handlers
-    virtual bool is_dap4_only_type();
-    virtual bool is_dap2_only_type();
-#endif
     virtual bool synthesized_p();
     virtual void set_synthesized_p(bool state);
 
@@ -292,11 +231,16 @@ public:
     virtual AttrTable &get_attr_table();
     virtual void set_attr_table(const AttrTable &at);
 
+    // DAP4 attributes
+    virtual D4Attributes *attributes();
+    virtual void set_attributes(D4Attributes *);
+    virtual void set_attributes_nocopy(D4Attributes *);
+
     virtual bool is_in_selection();
     virtual void set_in_selection(bool state);
 
     virtual void set_parent(BaseType *parent);
-    virtual BaseType *get_parent();
+    virtual BaseType *get_parent() const;
 
     virtual void transfer_attributes(AttrTable *at);
 
@@ -338,12 +282,16 @@ public:
     virtual BaseType *var(const string &name, btp_stack &s);
 
     virtual void add_var(BaseType *bt, Part part = nil);
+    virtual void add_var_nocopy(BaseType *bt, Part part = nil);
 
     virtual bool read();
 
     virtual bool check_semantics(string &msg, bool all = false);
 
     virtual bool ops(BaseType *b, int op);
+    virtual bool d4_ops(BaseType *b, int op);
+
+    virtual unsigned int width(bool constrained = false) const;
 
     virtual void print_decl(FILE *out, string space = "    ",
                             bool print_semi = true,
@@ -363,6 +311,8 @@ public:
 
     virtual void print_xml_writer(XMLWriter &xml, bool constrained = false);
 
+    virtual void print_dap4(XMLWriter &xml, bool constrained = false);
+
     /** @name Abstract Methods */
     //@{
 #if 0
@@ -379,8 +329,6 @@ public:
 	@brief Returns the size of the class instance data. */
     virtual unsigned int width(bool constrained = false) = 0;
 #endif
-    virtual unsigned int width(bool constrained = false);
-
     /** Reads the class data into the memory referenced by <i>val</i>.
 	The caller should either allocate enough storage to <i>val</i>
 	to hold the class data or set \c *val to null. If <i>*val</i>
@@ -440,7 +388,9 @@ public:
         to/from a stream.
 
         This method is defined by the various data type classes. It calls the
-        read() abstract method.
+        read() abstract method. Unlike serialize(), this method does not
+        clear the memory use to hold the data values, so the caller should
+        make sure to delete the DDS or the variable as soon as possible.
 
         @param eval Use this as the constraint expression evaluator.
         @param dds The Data Descriptor Structure object corresponding
@@ -449,9 +399,12 @@ public:
     virtual void intern_data(ConstraintEvaluator &eval, DDS &dds);
 
     /** Sends the data from the indicated (local) dataset through the
-	connection identified by the <i>sink</i> parameter. If the
+	connection identified by the Marshaller parameter. If the
 	data is not already incorporated into the DDS object, read the
-	data from the dataset.
+	data from the dataset. Once the data are sent (written to the
+	Marshaller), they are deleted from the object and the object
+	state is reset so that they will be read again if the read()
+	method is called.
 
 	This function is only used on the server side of the
 	client/server connection, and is generally only called from
@@ -459,24 +412,111 @@ public:
 	implementation; each datatype child class supplies its own
 	implementation.
 
-	@brief Move data to the net.
+	@brief Move data to the net, then remove them from the object.
 
-        @param eval Use this as the constraint expression evaluator.
+    @param eval Use this as the constraint expression evaluator.
 	@param dds The Data Descriptor Structure object corresponding
 	to this dataset. See <i>The DODS User Manual</i> for
 	information about this structure.
 	@param m A marshaller used to serialize data types
 	@param ce_eval A boolean value indicating whether to evaluate
 	the DODS constraint expression that may accompany this
-	dataset. The constraint expression is stored in <i>dds</i>.
+	dataset. The constraint expression is stored in the <i>dds</i>.
 	@return This method always returns true. Older versions used
 	the return value to signal success or failure.
 
+	@note We changed the default behavior of this method so that it
+	calls BaseType::clear_local_data() once the values are sent. This,
+	combined with the behavior that read() is called by this method
+	just before data are sent, means that data for any given variable
+	remain in memory for the shortest time possible. Furthermore, since
+	variables are serialized one at a time, no more than one variable's
+	data will be in memory at any given time when using the default
+	behavior. Some code - code that uses intern_data() or server functions -
+	might alter this default behavior. Only Array (i.e. Vector), Sequence,
+	D4Sequence and D4Opaque types actually hold data in dynamically allocated
+	memory, so sonly those types have the new/changed behavior.
+	This change was made on 7/5/15.
+
 	@exception InternalErr.
 	@exception Error.
 	@see DDS */
-    virtual bool serialize(ConstraintEvaluator &eval, DDS &dds,
-			   Marshaller &m, bool ce_eval = true) = 0;
+    virtual bool serialize(ConstraintEvaluator &eval, DDS &dds, Marshaller &m, bool ce_eval = true);
+
+#if 0
+    /**
+     * Provide a way to get the old behavior of serialize() - calling this
+     * method will serialize the BaseType object's data but _not_ delete its
+     * data storage.
+     *
+     * @note This method's behavior differs only for Array (i.e. Vector), Sequence,
+     * D4Sequence and D4Opaque types; the other types do not use dynamic memory to
+     * hold data values.
+     *
+     * @param eval Use this as the constraint expression evaluator.
+     * @param dds The Data Descriptor Structure object corresponding
+     * to this dataset. See <i>The DODS User Manual</i> for
+     * information about this structure.
+     * @param m A marshaller used to serialize data types
+     * @param ce_eval A boolean value indicating whether to evaluate
+     * the DODS constraint expression that may accompany this
+     * @return This method always returns true. Older versions used
+     * the return value to signal success or failure.
+     * @param
+     */
+    virtual bool serialize_no_release(ConstraintEvaluator &eval, DDS &dds, Marshaller &m, bool ce_eval = true) {
+        return serialize(eval, dds, m, ce_eval);
+    }
+#endif
+
+    /**
+     * @brief include the data for this variable in the checksum
+     * DAP4 includes a checksum with every data response. This method adds the
+     * variable's data to that checksum.
+     * @param checksum A Crc32 instance that holds the current checksum.
+     */
+    virtual void compute_checksum(Crc32 &checksum) = 0;
+
+    virtual void intern_data(/*Crc32 &checksum, DMR &dmr, ConstraintEvaluator &eval*/);
+
+    /**
+     * @brief The DAP4 serialization method.
+     * Serialize a variable's values for DAP4. This does not write the DMR
+     * persistent representation but does write that part of the binary
+     * data blob that holds a variable's data. Once a variable's data are
+     * serialized, that memory is reclaimed (by calling BaseType::clear_local_data())
+     *
+     * @param m
+     * @param dmr
+     * @param eval
+     * @param filter True if there is one variable that should be 'filtered'
+     * @exception Error or InternalErr
+     */
+    virtual void serialize(D4StreamMarshaller &m, DMR &dmr, bool filter = false);
+
+#if 0
+    /**
+     * @brief Variation on the DAP4 serialization method - retain data after serialization
+     * Serialize a variable's values for DAP4. This does not write the DMR
+     * persistent representation but does write that part of the binary
+     * data blob that holds a variable's data. Once a variable's data are
+     * serialized, that memory is reclaimed (by calling BaseType::clear_local_data())
+     *
+     * @note This version does not delete the storage of Array, D4Sequence or
+     * D4Opaque variables, as it the case with serialize(). For other types,
+     * this method and serialize have the same beavior (since those types do
+     * not us dynamic memory to hold data values).
+     *
+     * @param m
+     * @param dmr
+     * @param eval
+     * @param filter True if there is one variable that should be 'filtered'
+     * @exception Error or InternalErr
+     */
+    virtual void serialize_no_release(D4StreamMarshaller &m, DMR &dmr, bool filter = false) {
+        serialize(m, dmr, filter);
+    }
+#endif
 
     /** Receives data from the network connection identified by the
 	<tt>source</tt> parameter. The data is put into the class data
@@ -502,7 +542,15 @@ public:
 	@exception Error when a problem reading from the UnMarshaller is
 	found.
 	@see DDS */
-    virtual bool deserialize(UnMarshaller &um, DDS *dds, bool reuse = false) = 0;
+    virtual bool deserialize(UnMarshaller &um, DDS *dds, bool reuse = false);
+
+    /**
+     * The DAP4 deserialization method.
+     * @param um
+     * @param dmr
+     * @exception Error or InternalErr
+     */
+    virtual void deserialize(D4StreamUnMarshaller &um, DMR &dmr);
 
     /** Prints the value of the variable, with its declaration. This
 	function is primarily intended for debugging DODS
@@ -520,7 +568,7 @@ public:
 	variable declaration is printed as well as the value. */
 
     virtual void print_val(FILE *out, string space = "",
-                           bool print_decl_p = true) = 0;
+                           bool print_decl_p = true);
 
     /** Prints the value of the variable, with its declaration. This
 	function is primarily intended for debugging DODS
