@@ -62,19 +62,22 @@
 #include "DapObj.h"
 #endif
 
+#if 0
 #ifndef KEYWORDS_H_
 #include "Keywords2.h"
 #endif
+#endif
 
 #ifndef XMLWRITER_H_
 #include "XMLWriter.h"
 #endif
 
-using std::cout;
-
 namespace libdap
 {
 
+bool has_dap2_attributes(BaseType *btp);
+bool has_dap2_attributes(AttrTable &a);
+
 /** The DAP2 Data Descriptor Object (DDS) is a data structure used by
     the DAP2 software to describe datasets and subsets of those
     datasets.  The DDS may be thought of as the declarations for the
@@ -201,11 +204,11 @@ private:
     int d_timeout;              // alarm time in seconds. If greater than
                                 // zero, raise the alarm signal if more than
                                 // d_timeout seconds are spent reading data.
+#if 0
     Keywords d_keywords;	    // Holds keywords parsed from the CE
+#endif
 
-    long d_max_response_size;   // In bytes...
-
-
+    uint64_t d_max_response_size_kb;   // In kilobytes...
 
     friend class DDSTest;
 
@@ -256,7 +259,7 @@ public:
 
     virtual AttrTable &get_attr_table();
 
-    string filename();
+    string filename() const;
     void filename(const string &fn);
 
     /// Get the DAP major version as sent by the client
@@ -275,7 +278,9 @@ public:
     /// @deprecated
     void set_dap_version(double d);
 
-    Keywords &get_keywords() { return d_keywords; }
+#if 0
+    Keywords &get_keywords() {return d_keywords;}
+#endif
 
     /// Get the URL that will return this DDS/DDX/DataThing
     string get_request_xml_base() const { return d_request_xml_base; }
@@ -289,17 +294,42 @@ public:
     /// Set the namespace for this DDS/DDX object/response
     void set_namespace(const string &ns) { d_namespace = ns; }
 
-    /// Get the maximum response size, in KB. Zero indicates no limit.
-    long get_response_limit() { return d_max_response_size; }
+    /// Get the maximum response size, in bytes. Zero indicates no limit.
+    long get_response_limit() { return (long)(d_max_response_size_kb * 1024); }
 
-    /** Set the maximum response size. Zero is the default value. The size
-        is given in kilobytes.
-        @param size The maximum size of the response in kilobytes. */
-    void set_response_limit(long size) { d_max_response_size = size * 1024; }
+    /**
+     * @brief The maximum allowed response size, in kilobytes. Zero indicates no limit (default).
+     * @return The maximum allowed response size, in kilobytes. Zero indicates no limit (default).
+     */
+    uint64_t get_response_limit_kb() const { return d_max_response_size_kb; }
 
-    /// Get the estimated response size.
+    /**
+     * Set the maximum response size. Zero is the default value. The size
+     * is given in kilobytes (but stored internally as the number of bytes).
+     * @param size The maximum size of the response in kilobytes.
+     */
+    void set_response_limit(long size) { d_max_response_size_kb = size; }
+
+    /**
+     * @brief Set the maximum response size, in kilobytes.
+     * The size is given in kilobytes..
+     * @param size The maximum size of the response in kilobytes.
+     */
+    void set_response_limit_kb(uint64_t size) {  d_max_response_size_kb = size; }
+
+    /**
+     * @return Returns true if the total data bytes requested exceeds the set limit, false otherwise.
+     */
+    bool too_big() {
+        return d_max_response_size_kb != 0 && get_request_size_kb(true) > d_max_response_size_kb;
+    }
+
+    /// Get the estimated response size in bytes.
     int get_request_size(bool constrained);
 
+    /// Get the estimated response size in kilobytes.
+    uint64_t get_request_size_kb(bool constrained);
+
     string container_name() ;
     void container_name( const string &cn ) ;
     Structure *container() ;
@@ -316,10 +346,18 @@ public:
 
     /// Return an iterator to the first variable
     Vars_iter var_begin();
+#if 0
+    /// Return a const iterator.
+    Vars_citer var_cbegin() const { return vars.cbegin(); }
+#endif
     /// Return a reverse iterator
     Vars_riter var_rbegin();
     /// Return an iterator
     Vars_iter var_end();
+#if 0
+    /// Return a const iterator
+    Vars_citer var_cend() const { return vars.cend(); }
+#endif
     /// Return a reverse iterator
     Vars_riter var_rend();
     /// Get an iterator
@@ -334,10 +372,16 @@ public:
     /// Removes a range of variables from the DDS.
     void del_var(Vars_iter i1, Vars_iter i2);
 
+    /** @name DDS_timeout
+     *  Old deprecated DDS timeout code
+     *  @deprecated
+     */
+    ///@{
     void timeout_on();
     void timeout_off();
     void set_timeout(int t);
     int get_timeout();
+    ///@}
 
     // These parse the DAP2 curly-brace document and make a C++ object.
     void parse(string fname);
@@ -361,6 +405,8 @@ public:
     void print_dmr(ostream &out, bool constrained);
 
     void print_das(ostream &out);
+    DAS *get_das();
+    void get_das(DAS *das);
 
     void mark_all(bool state);
     bool mark(const string &name, bool state);
