@@ -34,12 +34,9 @@
 
 #include "config.h"
 
-static char rcsid[] not_used =
-    {"$Id: Error.cc 27192 2013-10-01 18:29:06Z jimg $"
-    };
-
 #include <cstdio>
 #include <cassert>
+#include <utility>
 
 #include "Error.h"
 #include "parser.h"
@@ -69,46 +66,10 @@ static const char *err_messages[] = {
     "Malformed expression",
     "No authorization",
     "Cannot read file",
-    "Cannot read file"
+    "Not Implemented",
+    ""
 };
 
-/** Specializations of Error should use this to set the error code and
-    message. */
-Error::Error() : _error_code(undefined_error), _error_message("")
-{}
-
-/** Create an instance with a specific code and message string. This ctor
-    provides a way to to use any code and string you'd like. The code can be
-    one of the standard codes or it may be specific to your server. Thus a
-    client which can tell it's dealing with a specific type of server can use
-    the code accordingly. In general, clients simply show the error message
-    to users or write it to a log file.
-
-    @param ec The error code
-    @param msg The error message string. */
-Error::Error(ErrorCode ec, string msg)
-        : _error_code(ec), _error_message(msg)
-{}
-
-/** Create an instance with a specific message. The error code is set to \c
-    unknown_error.
-
-    @param msg The error message.
-    @see ErrorCode */
-Error::Error(string msg)
-        : _error_code(unknown_error), _error_message(msg)
-{}
-
-Error::Error(const Error &copy_from)
-        : _error_code(copy_from._error_code),
-        _error_message(copy_from._error_message)
-{
-}
-
-Error::~Error()
-{
-}
-
 Error &
 Error::operator=(const Error &rhs)
 {
@@ -120,6 +81,9 @@ Error::operator=(const Error &rhs)
         _error_code = rhs._error_code;
         _error_message = rhs._error_message;
 
+        d_file = rhs.d_file;
+        d_line  = rhs.d_line;
+
         assert(this->OK());
 
         return *this;
@@ -280,14 +244,14 @@ Error::get_error_message() const
 {
     assert(OK());
 
-    return string(_error_message);
+    return {_error_message};
 }
 
 /** Set the error message. */
 void
 Error::set_error_message(string msg)
 {
-    _error_message = msg;
+    _error_message = std::move(msg);
 }
 
 } // namespace libdap
