@@ -36,16 +36,15 @@
 #define _operators_h
 
 #include "GNURegex.h"  // GNU Regex class used for string =~ op.
-#include "parser.h"  // for ID_MAX
+
 #include "ce_expr.tab.hh"
 
-using namespace std;
+#pragma GCC diagnostic ignored "-Wsign-compare"
 
 namespace libdap {
 
 /** Compare two numerical types, both of which are either signed or unsigned.
- This class is one implementation of the comparison policy used by
- rops.
+ This class is one implementation of the comparison policy used by rops.
 
  @see rops
  @see USCmp
@@ -53,6 +52,8 @@ namespace libdap {
 template<class T1, class T2>
 bool Cmp(int op, T1 v1, T2 v2)
 {
+	DBGN(cerr << __PRETTY_FUNCTION__ << v1 << " " << op << " " << v2 << endl);
+
     switch (op) {
         case SCAN_EQUAL:
             return v1 == v2;
@@ -67,17 +68,27 @@ bool Cmp(int op, T1 v1, T2 v2)
         case SCAN_LESS_EQL:
             return v1 <= v2;
         case SCAN_REGEXP:
-            throw Error("Regular expressions are supported for strings only.");
+            throw Error(malformed_expr, "Regular expressions are supported for strings only.");
         default:
-            throw Error("Unrecognized operator.");
+            throw Error(malformed_expr, "Unrecognized operator.");
     }
 }
 
+#if 0
 template<class T>
 static inline unsigned long long dap_floor_zero(T i)
 {
     return (unsigned long long) ((i < 0) ? 0 : i);
 }
+#endif
+
+#if 0
+// Hack
+template<class T>
+static inline T dap_floor_zero(T i)
+{
+    return i;
+}
 
 /** Compare two numerical types, the first one unsigned and the second
  signed. If the signed argument is negative, zero is used in the
@@ -90,6 +101,8 @@ static inline unsigned long long dap_floor_zero(T i)
 template<class UT1, class T2>
 bool USCmp(int op, UT1 v1, T2 v2)
 {
+	DBGN(cerr << __PRETTY_FUNCTION__ << v1 << " " << op << " " << v2 << endl);
+
     switch (op) {
         case SCAN_EQUAL:
             return v1 == dap_floor_zero<T2>(v2);
@@ -104,9 +117,9 @@ bool USCmp(int op, UT1 v1, T2 v2)
         case SCAN_LESS_EQL:
             return v1 <= dap_floor_zero<T2>(v2);
         case SCAN_REGEXP:
-            throw Error("Regular expressions are supported for strings only.");
+            throw Error(malformed_expr, "Regular expressions are supported for strings only.");
         default:
-            throw Error("Unrecognized operator.");
+            throw Error(malformed_expr, "Unrecognized operator.");
     }
 }
 
@@ -125,6 +138,8 @@ bool USCmp(int op, UT1 v1, T2 v2)
 template<class T1, class UT2>
 bool SUCmp(int op, T1 v1, UT2 v2)
 {
+	DBGN(cerr << __PRETTY_FUNCTION__ << v1 << " " << op << " " << v2 << endl);
+
     switch (op) {
         case SCAN_EQUAL:
             return dap_floor_zero<T1>(v1) == v2;
@@ -139,15 +154,15 @@ bool SUCmp(int op, T1 v1, UT2 v2)
         case SCAN_LESS_EQL:
             return dap_floor_zero<T1>(v1) <= v2;
         case SCAN_REGEXP:
-            throw Error("Regular expressions are supported for strings only.");
+            throw Error(malformed_expr, "Regular expressions are supported for strings only.");
         default:
-            throw Error("Unrecognized operator.");
+            throw Error(malformed_expr, "Unrecognized operator.");
     }
 }
+#endif
 
 /** Compare two string types.
- This class is one implementation of the comparison policy used by
- rops.
+ This class is one implementation of the comparison policy used by rops.
 
  @see rops */
 template<class T1, class T2>
@@ -167,11 +182,11 @@ bool StrCmp(int op, T1 v1, T2 v2)
         case SCAN_LESS_EQL:
             return v1 <= v2;
         case SCAN_REGEXP: {
-            Regex r(v2.c_str());
+            Regex r(v2);
             return r.match(v1.c_str(), v1.length()) > 0;
         }
         default:
-            throw Error("Unrecognized operator.");
+            throw Error(malformed_expr, "Unrecognized operator.");
     }
 }
 
