@@ -27,7 +27,14 @@
 #include <signal.h>
 #include <unistd.h>
 #include <sys/stat.h>
+
+#ifdef HAVE_UUID_UUID_H
 #include <uuid/uuid.h>  // used to build CID header value for data ddx
+#elif defined(HAVE_UUID_H)
+#include <uuid.h>
+#else
+#error "Could not find UUID library header"
+#endif
 
 #ifndef WIN32
 #include <sys/wait.h>
@@ -82,8 +89,8 @@ void ResponseBuilder::initialize()
     // Set default values. Don't use the C++ constructor initialization so
     // that a subclass can have more control over this process.
     d_dataset = "";
-    d_ce = "";
-    d_btp_func_ce = "";
+    d_dap2ce = "";
+    d_dap2_btp_func_ce = "";
     d_timeout = 0;
 
     d_default_protocol = DAP_PROTOCOL_VERSION;
@@ -105,7 +112,7 @@ ResponseBuilder::~ResponseBuilder()
  @return A string object that contains the constraint expression. */
 string ResponseBuilder::get_ce() const
 {
-    return d_ce;
+    return d_dap2ce;
 }
 
 /** Set the constraint expression. This will filter the CE text removing
@@ -120,7 +127,7 @@ string ResponseBuilder::get_ce() const
  */
 void ResponseBuilder::set_ce(string _ce)
 {
-    d_ce = www2id(_ce, "%", "%20");
+    d_dap2ce = www2id(_ce, "%", "%20");
 }
 
 /** The ``dataset name'' is the filename or other string that the
@@ -196,7 +203,7 @@ ResponseBuilder::split_ce(ConstraintEvaluator &eval, const string &expr)
     if (!expr.empty())
         ce = expr;
     else
-        ce = d_ce;
+        ce = d_dap2ce;
 
     string btp_function_ce = "";
     string::size_type pos = 0;
@@ -233,8 +240,8 @@ ResponseBuilder::split_ce(ConstraintEvaluator &eval, const string &expr)
     DBG(cerr << "Modified constraint: " << ce << endl);
     DBG(cerr << "BTP Function part: " << btp_function_ce << endl);
 
-    d_ce = ce;
-    d_btp_func_ce = btp_function_ce;
+    d_dap2ce = ce;
+    d_dap2_btp_func_ce = btp_function_ce;
 }
 
 #if 0
@@ -327,7 +334,7 @@ void ResponseBuilder::send_das(ostream &out, DDS &dds, ConstraintEvaluator &eval
     else {
         DBG(cerr << "Simple constraint" << endl);
 
-        eval.parse_constraint(d_ce, dds); // Throws Error if the ce doesn't parse.
+        eval.parse_constraint(d_dap2ce, dds); // Throws Error if the ce doesn't parse.
 
         if (with_mime_headers)
             set_mime_text(out, dods_das, x_plain, last_modified_time(d_dataset), dds.get_dap_version());
@@ -393,13 +400,13 @@ void ResponseBuilder::send_dds(ostream &out, DDS &dds, ConstraintEvaluator &eval
         }
 
         // Server functions might mark variables to use their read()
-        // methods. Clear that so the CE in d_ce will control what is
+        // methods. Clear that so the CE in d_dap2ce will control what is
         // sent. If that is empty (there was only a function call) all
         // of the variables in the intermediate DDS (i.e., the function
         // result) will be sent.
         fdds->mark_all(false);
 
-        eval.parse_constraint(d_ce, *fdds);
+        eval.parse_constraint(d_dap2ce, *fdds);
 
         if (with_mime_headers)
             set_mime_text(out, dods_dds, x_plain, last_modified_time(d_dataset), dds.get_dap_version());
@@ -414,7 +421,7 @@ void ResponseBuilder::send_dds(ostream &out, DDS &dds, ConstraintEvaluator &eval
     else {
         DBG(cerr << "Simple constraint" << endl);
 
-        eval.parse_constraint(d_ce, dds); // Throws Error if the ce doesn't parse.
+        eval.parse_constraint(d_dap2ce, dds); // Throws Error if the ce doesn't parse.
 
         if (with_mime_headers)
             set_mime_text(out, dods_dds, x_plain, last_modified_time(d_dataset), dds.get_dap_version());
@@ -475,25 +482,25 @@ void ResponseBuilder::send_data(ostream &data_stream, DDS &dds, ConstraintEvalua
     // If there are functions, parse them and eval.
     // Use that DDS and parse the non-function ce
     // Serialize using the second ce and the second dds
-    if (!d_btp_func_ce.empty()) {
+    if (!d_dap2_btp_func_ce.empty()) {
         DBG(cerr << "Found function(s) in CE: " << d_btp_func_ce << endl);
         string cache_token = "";
         DDS *fdds = 0;
 
         // The BES code caches the function result
-            eval.parse_constraint(d_btp_func_ce, dds);
+            eval.parse_constraint(d_dap2_btp_func_ce, dds);
             fdds = eval.eval_function_clauses(dds);
 
         DBG(fdds->print_constrained(cerr));
 
         // Server functions might mark variables to use their read()
-        // methods. Clear that so the CE in d_ce will control what is
+        // methods. Clear that so the CE in d_dap2ce will control what is
         // sent. If that is empty (there was only a function call) all
         // of the variables in the intermediate DDS (i.e., the function
         // result) will be sent.
         fdds->mark_all(false);
 
-        eval.parse_constraint(d_ce, *fdds);
+        eval.parse_constraint(d_dap2ce, *fdds);
 
         fdds->tag_nested_sequences(); // Tag Sequences as Parent or Leaf node.
 
@@ -516,7 +523,7 @@ void ResponseBuilder::send_data(ostream &data_stream, DDS &dds, ConstraintEvalua
 
 	DBG(cerr << "Simple constraint" << endl);
 
-	eval.parse_constraint(d_ce, dds); // Throws Error if the ce doesn't parse.
+	eval.parse_constraint(d_dap2ce, dds); // Throws Error if the ce doesn't parse.
 
 	dds.tag_nested_sequences(); // Tag Sequences as Parent or Leaf node.
 
