@@ -39,6 +39,7 @@
 #include <cassert>
 #include <cstring>
 #include <climits>
+#include <cstdlib>
 
 #include <ctype.h>
 #ifndef TM_IN_SYS_TIME
@@ -331,6 +332,9 @@ template<class T> static void extract_double_array_helper(Array * a, vector<doub
  * array using 'new double[n]' so delete[] MUST be used when you are done
  * the data.
  *
+ * @note Before you call this function, you must call read on the array,
+ * or load the array with values and set the read_p property.
+ *
  * @note Support added for DAP4.
  * @param a Extract value from this Array.
  * @param dest Put the values in this vector. A value-result parameter.
@@ -346,7 +350,7 @@ void extract_double_array(Array *a, vector<double> &dest)
         throw Error(malformed_expr, "The function requires a DAP numeric-type array argument.");
 
     if (!a->read_p())
-        throw InternalErr(__FILE__, __LINE__, string("The Array '") + a->name() + "'does not contain values.");
+        throw InternalErr(__FILE__, __LINE__, string("The Array '") + a->name() + "' does not contain values.");
 
     dest.resize(a->length());
 
@@ -521,7 +525,9 @@ libdap_root()
 extern "C" const char *
 libdap_version()
 {
-    return PACKAGE_VERSION;
+    // return PACKAGE_VERSION;
+    // Switched to CVER which is <version>-<build_number>
+    return CVER;
 }
 
 extern "C" const char *
@@ -803,8 +809,6 @@ bool is_simple_type(Type t)
     default:
         return false;
     }
-
-    return false;
 }
 
 /** @brief Returns true if the instance is a vector (i.e., array) type
@@ -848,8 +852,6 @@ bool is_vector_type(Type t)
     default:
         return false;
     }
-
-    return false;
 }
 
 /** @brief Returns true if the instance is a constructor (i.e., Structure,
@@ -891,8 +893,6 @@ bool is_constructor_type(Type t)
     default:
         return true;
     }
-
-    return false;
 }
 
 /**
@@ -1138,8 +1138,6 @@ int glob(const char *c, const char *s)
             break;
         }
     }
-
-    return 1;   // Should never get here; this quiets gcc's warning
 }
 
 /** @name Security functions */
@@ -1174,16 +1172,36 @@ bool pathname_ok(const string &path, bool strict)
 {
     if (path.length() > 255) return false;
 
+#if 0
+    // Make this use a const regex 12/1/21
     Regex name("[-0-9A-z_./]+");
     if (!strict) name = "[:print:]+";
+#endif
 
+    const Regex strict_name("[-0-9A-z_./]+");
+    const Regex relaxed_name("[:print:]+");
+
+#if 0
     string::size_type len = path.length();
-    int result = name.match(path.c_str(), len);
+#endif
+    int result;
+    if (strict)
+        result = strict_name.match(path);
+    else
+        result = relaxed_name.match(path);
+
+    return (result == path.length());
+
+#if 0
     // Protect against casting too big an uint to int
     // if LEN is bigger than the max int32, the second test can't work
+
+    // This makes no sense - len can never be > 255 given the test at the
+    // start of this function. jhrg 12/1/21
     if (len > INT_MAX || result != static_cast<int>(len)) return false;
 
     return true;
+#endif
 }
 
 //@}
